"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var PropTypes = require("prop-types");
/**
 * Default setting
 */
exports.DEFAULT_BEM_SETTING = {
    modifierDelimiter: '--',
    elementDelimiter: '__',
    bemIndicationElement: /:block\(([a-z]*)\)/gi,
    bemIndicationMod: /:element\(([a-z]*)\)/gi,
    bemIndicationSeparator: ':'
};
/**
 * Create class name with modifiers
 * @param {string} className
 * @param {string[]} modifiers
 * @param {object} setting
 * @return {string[]}
 */
function addModifiersToClassName(className, modifiers, setting) {
    if (modifiers === void 0) { modifiers = []; }
    if (setting === void 0) { setting = exports.DEFAULT_BEM_SETTING; }
    var _a = className.split(setting.elementDelimiter), block = _a[0], element = _a[1];
    var modifiersName = modifiers
        .filter(function (modifier) { return typeof modifier === 'string' && modifier.length; })
        .map(function (modifier) { return getStringBemInfo(modifier, setting); })
        .filter(function (bemModifierInfo) { return checkBemInfoCondition(bemModifierInfo, block, element); })
        .map(function (bemModifierInfo) { return "" + className + setting.modifierDelimiter + bemModifierInfo.className; });
    return [className].concat(modifiersName);
}
exports.addModifiersToClassName = addModifiersToClassName;
/**
 * Test bemInfo values by blockName and elementName
 * @param {object} bemInfo
 * @param {string} block
 * @param {string} element
 * @return {bool}
 */
function checkBemInfoCondition(bemInfo, block, element) {
    if (element === void 0) { element = ''; }
    if (bemInfo.blocksName.length > 0 && bemInfo.blocksName.indexOf(block) === -1) {
        return false;
    }
    if (bemInfo.elementsName.length > 0 && bemInfo.elementsName.indexOf(element) === -1) {
        return false;
    }
    return true;
}
exports.checkBemInfoCondition = checkBemInfoCondition;
/**
 * Create class name with modifiers
 * @param {string} className
 * @param {object} setting
 * @return {object}
 */
function getStringBemInfo(className, setting) {
    if (setting === void 0) { setting = exports.DEFAULT_BEM_SETTING; }
    var blocksName = [];
    var elementsName = [];
    var onlyClassName = className.split(setting.bemIndicationSeparator)[0];
    var match;
    // eslint-disable-next-line no-cond-assign
    while (match = setting.bemIndicationElement.exec(className)) {
        var blockName = match[1];
        blocksName.push(blockName);
    }
    // eslint-disable-next-line no-cond-assign
    while (match = setting.bemIndicationMod.exec(className)) {
        var elementName = match[1];
        elementsName.push(elementName);
    }
    return {
        className: onlyClassName,
        blocksName: blocksName,
        elementsName: elementsName
    };
}
exports.getStringBemInfo = getStringBemInfo;
/**
 * Converts many possible values to one format.
 * return empty array if bemValue is invalid
 * @param {string|object|string[]} bemValue
 * @return {string[]}
 */
function convertBemValueToArray(bemValue) {
    if (typeof bemValue === 'string') {
        return bemValue.split(' ');
    }
    if (typeof bemValue === 'object') {
        if (bemValue === null) {
            return [];
        }
        if (Array.isArray(bemValue)) {
            return bemValue;
        }
        return Object.keys(bemValue)
            .filter(function (key) {
            var value = bemValue[key];
            return typeof value === 'function' ? value() : value;
        });
    }
    return [];
}
exports.convertBemValueToArray = convertBemValueToArray;
function cleanUpProps(props) {
    props = __assign({}, props);
    delete props.bemName;
    delete props.bemMod;
    delete props.className;
    delete props.bemBlock;
    return props;
}
exports.cleanUpProps = cleanUpProps;
/**
 * Validator for bem value
 */
exports.PropTypesBemValue = PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.arrayOf(PropTypes.string),
    PropTypes.object
]);
/**
 * Validator for bem setting
 */
exports.PropTypesBemSetting = PropTypes.shape({
    elementDelimiter: PropTypes.string,
    modifierDelimiter: PropTypes.string,
    bemIndicationElement: PropTypes.object,
    bemIndicationMod: PropTypes.object,
    bemIndicationSeparator: PropTypes.string
});
